package FIMM::BioIO;

use strict;
use warnings;
use Carp;

use FIMM::BioIO::PeptideList::Mascot;
use FIMM::BioIO::PeptideList::Tabular;
use FIMM::Log qw(debug);

BEGIN {
    require Exporter;
    our $VERSION = 0.0;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(open_peptide_list
                        read_fasta);
}

=head1 NAME

FIMM::BioIO - functions for handling common bioinformatics files

=head1 SYNOPSIS

    use FIMM::BioIO qw(open_peptide_list read_fasta);

    $s = read_fasta("HUMAN.fasta");
    $name = $s->{A6NKF1}->{name};        # SAC31_HUMAN
    $desc = $s->{A6NKF1}->{description}; # SAC3 domain-containing protein 1
    $seq = $s->{A6NKF1}->{sequence};     # MAGRRAQTGSAPPRPAAPHP...

    # Use the factory function open_peptide_list()
    $peptides = open_peptide_list("mascot-list.csv")
        or die "Cannot open peptide list";
    while (($accessions, $sequence, $offsets) = $peptides->next()) {
        # process ...
    }

=head1 DESCRIPTION

This module provides functions for accessing FASTA files and peptide
lists.  FASTA files are ubiquituos for representing nucleotide and
amino acid sequences.  Peptide lists are produced by mass spectrometry
analysis tools.  This module tries to provide a uniform and file
format-agnostic interface to such files.

=head1 FUNCTIONS

=head2 read_fasta $file

Reads in a FASTA file $file.  Various FASTA files may differ in their
formatting.  This function assumes SwissProt or NCBI formatting.
Example records:

  >sp|A0A183|LCE6A_HUMAN Late cornified envelope protein 6A OS=Homo sapiens GN=LCE6A PE=2 SV=1
  MSQQKQQSWKPPNVPKCSPPQRSNPCLAPYSTPCGAPHSEGCHSSSQRPEVQKPRRARQK
  LRCLSRGTTYHCKEEECEGD

(SwissProt)

  >gi|157823857|ref|NP_001102725.1| sarcospan [Rattus norvegicus]
  MGRKPGPGAQELAEEEARTCCGCRFPLLLALLQLALGIAVTVLGFLMASISPSLLVRDTPFWAGIIVCVVAYLGLFMLCV

(NCBI)

Returns a hashref indexed by protein identifiers (C<A0A183> and
C<157823857> in the examples above).  Each value is a hashref with the
following keys:

=over

=item * C<name>: Short name (C<LCE6A_HUMAN> in the example above) or
RefSeq number (C<NP_001102725.1>).

=item * C<description>: Recommended name (C<Late cornified envelope
protein 6A> or C< sarcospan [Rattus norvegicus]> above).

=item * C<sequence>: Sequence data as a string of amino acids.
Newlines will be stripped.

=back

=cut

sub read_fasta {
    my $fasta_file = shift;
    open(my $seqfile, "<", $fasta_file) or croak "cannot open $fasta_file";

    local $/ = "\n>";
    my $sequences;
    while (<$seqfile>) {
        chomp;

        my @tokens = split /\|/;

        if ($tokens[0] =~ />?sp/) {
            # SwissProt / TrEMBL
            my $id = $tokens[1];
            my ($meta, @seq_lines) = split /\n/, $tokens[-1];
            if ($meta =~ /(\w+) (.+) (OS=)/ != 1) {
                confess "fishy chunk: $_";
            }
            $sequences->{$id}->{name} = $1;
            $sequences->{$id}->{description} = $2;
            @seq_lines or croak "no sequence found for $id";
            $sequences->{$id}->{sequence} = join '', @seq_lines;
        } elsif ($tokens[0] =~ />?gi/) {
            # NCBI
            my ($id, $name) = ($tokens[1], $tokens[3]);
            my ($description, @seq_lines) = split /\n/, $tokens[-1];
            $sequences->{$id}->{name} = $name;
            $sequences->{$id}->{description} = $description;
            @seq_lines or croak "no sequence found for $id";
            $sequences->{$id}->{sequence} = join '', @seq_lines;
        }
    }

    return $sequences;
}

=head2 open_peptide_list $filename, @querycols

Factory function for building PeptideList objects.  This function
tries each known concrete PeptideList class constructor with the
arguments $filename and @querycols, and returns the first valid object
reference.  If none of the constuctors succeed, (C<undef>, I<error
message>) pair is returned.

=cut

sub open_peptide_list {
    my ($filename, @qcols) = @_;
    my ($list, $error);
    if (not -f $filename) {
        $error = "$filename does not exist";
    } else {
        $list = (FIMM::BioIO::PeptideList::Mascot->new($filename, @qcols)
                 || FIMM::BioIO::PeptideList::Tabular->new($filename, @qcols)
                 || undef);
        if (not $list) {
            $error = "cannot recognize '$filename' as valid peptide list";
        }
    }
    return ($list, $error);
}

=head1 AUTHOR

Petteri Hintsanen <petterih@iki.fi>

=head1 SEE ALSO

L<FIMM::BioIO::PeptideList>

=head1 COPYRIGHT AND LICENSE

Copyright 2013, 2015 Petteri Hintsanen

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1
