package FIMM::BioIO::PeptideList;

use strict;
use warnings;
use Carp;

use Object::Tiny qw{ filename fh };

=head1 NAME

FIMM::BioIO::PeptideList - base class for peptide list iteration

=head1 DESCRIPTION

FIMM::BioIO::PeptideList is an abstract class: it is not instantiable.
Concrete PeptideList classes are used to iterate over different types
of peptide lists in a uniform manner.  Each derived class is supposed
to handle a specific kind of list (eg. tab-separated or
comma-separated file), and return the protein accessions, the peptide
sequence and the post-translational modification sites for each
peptide found in the list.  See next() for details.

=head1 FUNCTIONS

=head2 new $filename, @querycols

Opens the peptide list file $filename and sets the file name and file
handle members accordingly.

Derived classes overriding this new() can additionally take a list of
field names (@querycols) whose values are returned for each peptide
after a call to next().  This particular new() ignores @querycols.

If $filename cannot be opened for some reason, a warning is issued
with carp() and C<undef> is returned.  Otherwise the function returns
a reference to the new object.

=cut

sub new {
    my $class = shift;
    if ($class eq "FIMM::BioIO::PeptideList") {
        carp "FIMM::BioIO::PeptideList is an abstract class";
        return;
    }
    my $self = $class->SUPER::new(@_);

    $self->{filename} = shift;
    if (not $self->{filename}) {
        carp "Must give peptide list filename";
        return;
    }

    unless (open($self->{fh}, "<", $self->{filename})) {
        carp "Cannot open $self->{filename}: $!";
        return;
    }

    return $self;
}

=head2 filename

Accessor for the associated list file name.

=head2 fh

Accessor for the associated list file handle.  Call to new() opens the
handle, so it should be always a valid file handle if the object is
valid.

=head2 next

This method iterates over the listed peptides.  It returns a following
hashref:

  $peptide = {
    accessions =>  [ accession... ] # A list of protein accession numbers
                              # for the peptide.
    sequence => string        # The amino acid sequence of the peptide.

    mods => {                 # Post-translational modifications.
      type => [ integer... ]  # Modification type, either "phospho"
                              # or "acetyl", and the corresponding site offsets
                              # (one-based) from the beginning of the *peptide*.
    },

    query_cols => {           # Values for each query column (given
                              # for new() in the derived classes), if any.
      col1 => value,          # Value in the first query column.
      col2 => value,          # Value in the second query column.
      ...
    }
  }

If there are no more peptides, the function returns undef.

=cut

sub next {
    croak "not implemented!";
}

=head2 type

Returns the type (string) of the list.

=cut

sub type {
    croak "not implemented!";
}

=head2 line

Returns the current line number being processed.

=cut

sub line {
    my $self = shift;
    return $self->{fh}->input_line_number();
}

=head1 AUTHOR

Petteri Hintsanen <petterih@iki.fi>

=head1 SEE ALSO

L<FIMM::BioIO::PeptideList::Mascot>,
L<FIMM::BioIO::PeptideList::Tabular>

=head1 COPYRIGHT AND LICENSE

Copyright 2013, 2015 Petteri Hintsanen

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1
