package FIMM::BioIO::PeptideList::Mascot;
use parent FIMM::BioIO::PeptideList;

use strict;
use warnings;
use Carp;
use List::Util qw(first);
use Text::CSV;

use FIMM::Log qw(debug);

=head1 NAME

FIMM::BioIO::PeptideList::Mascot - process Mascot-like peptide lists

=head1 DESCRIPTION

See the base class L<FIMM::BioIO::PeptideList>.

=cut

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_) or return;
    shift;                      # Discard filename from arguments.

    $self->{csv} = Text::CSV->new({allow_loose_quotes => 1});

    # Look for variable modificatios section.
    my $fh = $self->{fh};
    while (<$fh>) {
        $self->{csv}->parse($_);
        my @fields = $self->{csv}->fields();
        if ($fields[0] =~ /Variable modifications/i) {
            $self->parse_ptm_ids();
            last;
        }
    }

    # Look for the header line.
    while (<$fh>) {
        $self->{csv}->parse($_);
        my @fields = $self->{csv}->fields();
        if ($fields[0] =~ /Protein hits/i) {
            if ($self->parse_header_line(@_)) {
                return $self;
            } else {
                return undef;
            }
        }
    }

    # Give up.
    return undef;
}

sub parse_ptm_ids {
    my $self = shift;
    debug "parsing modification identifiers";
    my $fh = $self->{fh};
  PTM: while (<$fh>) {
        $self->{csv}->parse($_);
        my @fields = $self->{csv}->fields();
        next PTM unless ($fields[0] =~ /Identifier/i);
        while (<$fh>) {
            $self->{csv}->parse($_);
            my ($id, $mod) = $self->{csv}->fields();
            last PTM if (!defined($id) || $id eq "");
            die "id '$id' mod  '$mod'" if (not ($id)  && not ($mod));
            if ($mod =~ /acetyl/i) { 
                debug("found acetylation id $id");
                push @{$self->{acetyl_ids}}, $id;
            } elsif ($mod =~ /phospho/i) {
                debug("found phospho id $id");
                push @{$self->{phospho_ids}}, $id;
            }
        }
    }

    if ($self->{phospho_ids}) {
        $self->{phospho_pattern} = join '|', @{$self->{phospho_ids}};
        debug("phospho pattern is $self->{phospho_pattern}");
    }
    if ($self->{acetyl_ids}) {
        $self->{acetyl_pattern} = join '|', @{$self->{acetyl_ids}};
        debug("acetyl pattern is $self->{acetyl_pattern}");
    }

    debug "done parsing ptm identifiers, current line $.";
}

sub parse_header_line {
    my ($self, @qcols) = @_;
    debug("parsing header");
    # Skip empty lines, if any.
    my $fh = $self->{fh};
    while (<$fh>) {
        last if /^prot_hit_num/;
    }
    $self->{csv}->parse($_) or return undef;
    debug "found header at line $.";
    my @header = $self->{csv}->fields();
    $self->{accession_index} = first { $header[$_] =~ /prot_acc/ } 0..$#header;
    $self->{pep_seq_index} = first { $header[$_] =~ /pep_seq/ } 0..$#header;
    $self->{pep_mod_index} = first { $header[$_] =~ /pep_var_mod_pos/ } 0..$#header;

    # Ensure that the required fields have been found.
    unless ($self->{accession_index}
            && $self->{pep_seq_index}
            && $self->{pep_mod_index}) {
        return undef;
    }

    debug("acc index @ $self->{accession_index}");
    debug("seq index @ $self->{pep_seq_index}");
    debug("mod index @ $self->{pep_mod_index}");

    # Search for query columns.
    foreach my $qcol (@qcols) {
        my $i = first { $header[$_] =~ /$qcol/ } 0..$#header;
        if ($i) {
            debug("query column $qcol is at index $i");
            $self->{qcols}->{$qcol} = $i;
        };
    }

    return $self;
}

sub type {
    my $self = shift;
    return "Mascot";
}

sub next {
    my $self = shift;
    my $fh = $self->{fh};
  LINE: while (<$fh>) {
        $self->{csv}->parse($_) or return undef;
        my @fields = $self->{csv}->fields();
        my ($accession, $seq, $modstr) =
          ($fields[$self->{accession_index}],
           $fields[$self->{pep_seq_index}],
           $fields[$self->{pep_mod_index}]);
        unless (defined($accession || $seq || $modstr)) {
            carp "suspicious line $. in $self->{filename}, skipping";
            next LINE;
        }

        if (not $accession) {
            $accession = $self->{last_accession};
        } else {
            $accession =~ s/^"?(.*)"?$/$1/;
            $accession = (split /\|/, $accession)[0];
            next LINE if ($accession =~ /CON$|^REV|REV$/);
            $self->{last_accession} = $accession;
        }

        $seq =~ s/^"?(.*)"?$/$1/;
        $modstr =~ s/^"?(.*)"?$/$1/;

        my %qcols;
        while (my ($qcol, $index) = each %{$self->{qcols}}) {
            if (not defined $fields[$index]) {
                carp "suspicious line $. in $self->{filename}, skipping";
                next LINE;
            }
            $qcols{$qcol} = $fields[$index];
            $qcols{$qcol} =~ s/^"?(.*)"?$/$1/;
        }

        my $mods = $self->parse_ptms($modstr) if ($modstr);

        return { accessions => [ $accession ],
                 sequence => uc($seq),
                 mods => $mods,
                 query_cols => \%qcols };
    }

    return undef;
}

sub parse_ptms {
    my ($self, $mods) = @_;
    my $parsed_mods;
    if ($self->{acetyl_pattern}) {
        my $pat = $self->{acetyl_pattern};
        while ($mods =~ /$pat/g) {
            debug("found acetylation at $-[0] - 1");
            push @{$parsed_mods->{acetyl}}, $-[0] - 1;
        }
    }
    pos($mods) = undef;
    if ($self->{phospho_pattern}) {
        my $pat = $self->{phospho_pattern};
        while ($mods =~ /$pat/g) {
            debug("found phosphorylation at $-[0] - 1");
            push @{$parsed_mods->{phospho}}, $-[0] - 1;
        }
    }
    return $parsed_mods;
}

=head1 AUTHOR

Petteri Hintsanen <petterih@iki.fi>

=head1 SEE ALSO

L<FIMM::BioIO::PeptideList>

=head1 COPYRIGHT AND LICENSE

Copyright 2013, 2014, 2015 Petteri Hintsanen

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1
