package FIMM::BioIO::PeptideList::Tabular;
use parent FIMM::BioIO::PeptideList;

use strict;
use warnings;
use Carp;
use List::Util qw(first);

use FIMM::Log qw(debug);

=head1 NAME

FIMM::BioIO::PeptideList::Tabular - process tabular peptide lists

=head1 DESCRIPTION

See the base class L<FIMM::BioIO::PeptideList>.

=cut

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_) or return;

    shift;
    my @qcols = @_;

    my $fh = $self->{fh};
    while (<$fh>) {
        chomp;
        # Weak heuristic: tab-separated header line must be the first
        # non-empty line in the input file, and it must have at least
        # three fields.  Otherwise, give up.
        $_ or next;
        my @header = split /\t/;
        return undef unless @header >= 3;

        # Look for required fields.
        $self->{accession_index} = first { $header[$_] =~ /(\bAccessions\b|\bProtein Group Accessions\b)/ } 0..$#header;
        $self->{pep_seq_index} = first { $header[$_] =~ /Sequence/ } 0..$#header;
        $self->{pep_mod_index} = first { $header[$_] =~ /Modifications/ } 0..$#header;

        # Ensure that the required fields have been found.
        unless ($self->{accession_index}
                && $self->{pep_seq_index}
                && $self->{pep_mod_index}) {
            return undef;
        }

        debug("acc index @ $self->{accession_index}");
        debug("seq index @ $self->{pep_seq_index}");
        debug("mod index @ $self->{pep_mod_index}");

        # Search for query columns.
        foreach my $qcol (@qcols) {
            my $i = first { $header[$_] =~ /$qcol/ } 0..$#header;
            if ($i) {
                debug("query column $qcol is at index $i");
                $self->{qcols}->{$qcol} = $i;
            };
        }

        return $self;
    }

    return undef;
}

sub type {
    my $self = shift;
    return "tabular";
}

sub next {
    my $self = shift;
    my $fh = $self->{fh};
    LINE: while (<$fh>) {
        my @tokens = split /\t/;
        my ($accession, $seq, $modstr) =
          ($tokens[$self->{accession_index}],
           $tokens[$self->{pep_seq_index}],
           $tokens[$self->{pep_mod_index}]);
        unless (defined($accession || $seq || $modstr)) {
            carp "suspicious line $. in $self->{filename}, skipping\n";
            next LINE;
        }

        my @accessions = split /;\s*/, $accession;
        my $naccs = @accessions;
        # TODO: diagnostics?
        @accessions = grep { $_ !~ /CON$|^REV|REV$/ } @accessions;
        @accessions = map {
            my @tokens = (split /\|/);
            if (@tokens == 3) { $tokens[1] }
            else { $tokens[0] }
        } @accessions;

        if ($naccs == 0 && @accessions == 0) {
            carp "failed to grep accessions from line $. in $self->{filename}, skipping\n", ;
            next LINE;
        }

        my %qcols;
        while (my ($qcol, $index) = each %{$self->{qcols}}) {
            if (not defined $tokens[$index]) {
                carp "suspicious line $. in $self->{filename}, skipping\n";
                next LINE;
            }
            my $value = $tokens[$index];
            $qcols{$qcol} = $value;
        }

        my $mods = $self->parse_ptms($modstr) if ($modstr);

        return { accessions => \@accessions,
                 sequence => uc($seq),
                 mods => $mods,
                 query_cols => \%qcols };
    }

    return undef;
}

sub parse_ptms {
    my ($self, $mods) = @_;
    my $parsed_mods;

    while ($mods =~ /Phospho\([A-Z]+\)@(\d+)/gc ||
           $mods =~ /[A-Z](\d+)\(Phospho\)/gc) {
        debug("found phosphorylation at $1");
        push @{$parsed_mods->{phospho}}, int($1);
    }

    pos($mods) = undef;

    while ($mods =~ /Acetyl\([A-Z]+\)@(\d+)/gc ||
           $mods =~ /[A-Z](\d+)\(Acetyl\)/gc) {
        debug("found acetylation at $1");
        push @{$parsed_mods->{acetyl}}, int($1);
    }

    return $parsed_mods;
}

=head1 AUTHOR

Petteri Hintsanen <petterih@iki.fi>

=head1 SEE ALSO

L<FIMM::BioIO::PeptideList>

=head1 COPYRIGHT AND LICENSE

Copyright 2013, 2015 Petteri Hintsanen

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1
