package FIMM::Log;

use strict;
use warnings;

BEGIN {
    require Exporter;
    our $VERSION = 0.0;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(debug logdie printlog saylog);
}

%FIMM::Log::debug = ();
%FIMM::Log::log_level = ( default => 1,
                          warning => 1,
                          error => 1,
                          fatal => 1 );
%FIMM::Log::log_prefix = ( default => undef,
                           warning => "*** warning",
                           error => "*** error",
                           fatal => "*** fatal error" );
$FIMM::Log::LOG = *STDERR;
$FIMM::Log::fatal_exit_code = 1;

=head1 NAME

FIMM::Log - simple logging and debug tracing

=head1 SYNOPSIS

    use FIMM::Log qw(debug saylog logdie);

    # This can be any filehandle, STDERR is the default.  Note that
    # this module does not do any flushing nor tamper with the
    # buffering of LOG.
    $FIMM::Log::LOG = *STDERR;

    # Prints "$0: debug: message" to STDERR if debugging is enabled
    # (see below).
    debug("message");

    # Prints $0: (timestamp) log message to $FIMM::Log::LOG.
    printlog("log message");

    # Equal to printlog("log message\n")
    saylog("log message");

    # Die with message, which goes always to STDERR also.
    logdie("we're outta here!");

    # Prints only if spurious_warning log level is true (see below).
    saylog("something nasty going on", "spurious_warning");

=head1 DESCRIPTION

This module provides straight-forward logging and debug tracing
facilities.  It gives some flexibility to control the amount of
output, albeit only at a global (program-wide) scale.

=head1 EXPORTED FUNCTIONS

=head2 debug $message

Prefixes $message with "$0: debug (PACKAGE): " and says it to STDERR,
if debugging is enabled.  Otherwise does nothing.

Debugging is enabled if $FIMM::Log::debug{PACKAGE} is true.  PACKAGE
is the package from which the call to debug() was made, as determined
by the caller() function.

=cut

sub debug {
    my $msg = shift;
    my $package = caller;
    return unless $FIMM::Log::debug{$package};
    print STDERR "$0: debug ($package): $msg\n";
}

=head2 printlog $message, $level

Prints $message to $FIMM::Log::LOG filehandle if
$FIMM::Log::log_level{$level} is true.  $message is prefixed with $0,
the current time stamp, and the value of
$FIMM::Log::log_prefix{$level} (if defined).

=cut

sub printlog {
    my ($msg, $level) = @_;
    $level or $level = "default";
    if ($FIMM::Log::log_level{$level}) {
        print $FIMM::Log::LOG "$0: " . timestamp() . " ";
        if ($FIMM::Log::log_prefix{$level}) {
            print $FIMM::Log::LOG "$FIMM::Log::log_prefix{$level}: ";
        }
        print $FIMM::Log::LOG "$msg";
    }
}

=head2 saylog $message, $level

Equivalent to C<printlog("$message\n", $level)>.

=cut

sub saylog {
    my ($msg, $level) = @_;
    printlog "$msg\n", $level;
}

=head2 logdie $message...

Logs $messages to $FIMM::Log::LOG and STDERR, and terminates the
program with exit code $FIMM::Log::fatal_exit_code (which is 1 by
default).

=cut

sub logdie {
    foreach (@_) {
        saylog $_, "fatal";
        print STDERR "$FIMM::Log::log_prefix{fatal}: $_\n"
          unless $FIMM::Log::LOG eq *STDERR;
    }
    exit $FIMM::Log::fatal_exit_code;
}

=head1 NON-EXPORTED FUNCTIONS

=head2 timestamp

Returns an ISO-like, locale-agnostic time stamp that is prefixed to
the log messages.

=cut

sub timestamp {
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
      localtime();
    $year += 1900;
    $mon += 1;
    my $timestamp = sprintf("(%d-%02d-%02d %02d:%02d:%02d)",
                            $year, $mon, $mday,
                            $hour, $min, $sec);
    return $timestamp;
}

=head1 CAVEATS

There are no optimizations whatsoever.  Each logging function makes
one or two hash lookups to determine what to output.  Calling lots of
logging functions in a very intense code may decrease performance,
even if the functions do not actually print anything.

There is no synchonization whatsoever.  Logging calls made from
separate threads get easily mixed.  You have to synchronize logging
yourself in multi-threaded programs.

=head1 AUTHOR

Petteri Hintsanen <petterih@iki.fi>

=head1 COPYRIGHT AND LICENSE

Copyright 2013, 2014, 2015 Petteri Hintsanen

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 SEE ALSO

L<Log::Message::Simple>, L<Log::Log4perl>

=cut

1
