package PhosFox::Clustering;

use strict;
use warnings;

use List::Util qw(min max);

use FIMM::Log qw(debug);

=head1 NAME

PhosFox::Clustering - group peptides which share the same PTMs

=head1 DESCRIPTION

This module contains peptide grouping code.  The module is part of
I<PhosFox>; it is not meant for general use.  Hence, it does not
export anything.  This POD explains the cluster data structure.

=head1 FUNCTIONS

=head2 cluster_peptides $listref, $modification

Divides peptides (hashrefs, see L<PhosFox::Input>) in $listref to
peptide clusters for reporting.  A peptide cluster represents all
peptides that share the same $modification at the exactly same sites.
$modification is either C<phospho> or C<acetyl>.

A cluster is laid out as follows:

  $cluster = {
    id => SCALAR,              # protein identifier for this cluster
    sites => [ INTEGER... ],   # modification sites of the peptides in this cluster
    acids => [ CHARACTER... ], # modified amino acids (as single letter characters)
    begin => INTEGER,          # largest starting index of the peptides in this cluster
    end => INTEGER,            # smallest ending index of the peptides in this cluster
    cases => HASH,             # number of case peptides in this cluster, indexed by search engine
    controls => INTEGER,       # number of control peptides in this cluster, indexed by search engine
    unique_case => BOOLEAN,    # the cluster has only case peptides
    unique_control => BOOLEAN  # the cluster has only control peptides
  }

The function returns a list of clusters.

=cut

sub cluster_peptides {
    my ($peptides, $mod) = @_;
    my @clusters;
  PEPTIDE: foreach my $peptide (@$peptides) {
        # If this peptide has no $mods, skip it.
        next PEPTIDE if (not $peptide->{mods}->{$mod});

        # Scan through existing clusters.
        my $cluster = undef;
      CLUSTER: foreach (@clusters) {
            # If this cluster has no $mod PTMs, skip it.
            next CLUSTER if (not $_->{sites});

            if (equals($peptide->{mods}->{$mod}->{sites}, $_->{sites})) {
                # This cluster has exactly the same sites as the
                # peptide.
                $cluster = $_;
                last CLUSTER;
            }
        }

        if ($cluster) {
            # debug "found cluster for protein $cluster->{id} with $mod PTMs @{$cluster->{mods}}";
            # Trim the cluster peptide, if necessary.
            $cluster->{begin} = max($cluster->{begin},
                                    $peptide->{begin});
            $cluster->{end} = min($cluster->{end},
                                  $peptide->{end});
        } else {
            # debug "new cluster @{$peptide->{mods}->{$mod}}";
            $peptide->{mods}->{$mod} or die <<'EOF';
No modifications for peptide.  This should never happen.
Please send a bug report to <petterih@iki.fi>
EOF
            $cluster =
              {
               id => $peptide->{id},
               sites => $peptide->{mods}->{$mod}->{sites},
               acids => $peptide->{mods}->{$mod}->{acids},
               begin => $peptide->{begin},
               end => $peptide->{end},
               cases => { },
               controls => { },
               unique_case => 0,
               unique_control => 0
              };

            push @clusters, $cluster;
        }

        $peptide->{engine} or die <<'EOF';
No engine for peptide.  This should never happen.
Please send a bug report to <petterih@iki.fi>
EOF
        $cluster->{cases}->{$peptide->{engine}}++ if $peptide->{case};
        $cluster->{controls}->{$peptide->{engine}}++ if $peptide->{control};
    }

    # Check for uniquely modified peptides.
    foreach my $cluster (@clusters) {      
        # print STDERR "protein id: $cluster->{id}\n";
        # print STDERR "case ";
        # print STDERR "$_ " foreach (keys (%{$cluster->{cases}}));
        # print STDERR "\n";
        # print STDERR "control ";
        # print STDERR "$_ " foreach (keys (%{$cluster->{controls}}));
        # print STDERR "\n";

        # print STDERR "case ";
        # print STDERR "$_ " foreach (values (%{$cluster->{cases}}));
        # print STDERR "\n";
        # print STDERR "control ";
        # print STDERR "$_ " foreach (values (%{$cluster->{controls}}));
        # print STDERR "\n";
        # print STDERR "---\n";

        my $ncases = grep { $_ > 0 } values(%{$cluster->{cases}});
        my $ncontrols = grep { $_ > 0 } values(%{$cluster->{controls}});

        if ($ncases > 0 && $ncontrols == 0) {
            $cluster->{unique_case} = 1;
        } else {
            $cluster->{unique_case} = 0;
        }

        if ($ncases == 0 && $ncontrols > 0) {
            $cluster->{unique_control} = 1;
        } else {
            $cluster->{unique_control} = 0;
        }

        if ($cluster->{unique_case} and $cluster->{unique_control}) {
            die <<'EOF';
Identical uniquely modified peptide detected both in case and control
sample.  This should never happen.
Please send a bug report to <petterih@iki.fi>.
EOF
        }
    }

    return \@clusters;
}

sub equals {
    my ($a, $b) = @_;
    return 0 if (!$a || !$b);
    return 0 if (@$a != @$b);
    my @sa = sort @$a;
    my @sb = sort @$b;
    for (my $i = 0; $i < @sa; $i++) {
        return 0 if $sa[$i] ne $sb[$i];
    }
    return 1;
}

=head1 SEE ALSO

phosfox(1), L<PhosFox::Input>

=cut

1
