package PhosFox::Configuration;

use strict;
use warnings;
use JSON;
use FIMM::Log qw(logdie saylog);

sub log_job_configuration;

$Configuration::DEFAULT_ACIDS = "STY";
$Configuration::DEFAULT_MODIFICATIONS = [ "phospho" ];

sub read_job_file {
    my $filename = shift;
    my $job_text;
    {
        local $/ = undef;
        open my $fh, '<', $filename
          or logdie "cannot open job file '$filename': $!\n";
        $job_text = <$fh>;
    }
    my $job;
    my $json = JSON->new->relaxed;
    eval { $job = $json->decode($job_text) };
    if ($@) {
        my ($offset) = $@ =~ /offset (\d+)/;
        unless ($offset) {
            logdie "malformed job file '$filename'";
        }
        my $start = rindex $job_text, "\n", $offset;
        my $end = index $job_text, "\n", $offset;
        logdie "cannot parse job file, stopped at line:\n"
          . substr $job_text, $start + 1, $end - $start - 1;
    }

    if (not defined($job->{fasta_files})) {
        logdie "no FASTA files specified";
    }

    if (not defined($job->{acids})) {
        $job->{acids} = $Configuration::DEFAULT_ACIDS;
    }

    # Validate input types.
    foreach my $input (@{$job->{inputs}}) {
        unless ($input->{type}) {
            logdie "missing input type";
        }
        unless ($input->{type} eq "case"
                or $input->{type} eq "control"
                or $input->{type} eq "quantified") {
            logdie("unknown input type '$input->{type}' was given\n"
                   . "input type must be 'case', 'control', or 'quantified'");
        }
    }

    # Validate quantification cutoffs.
    foreach my $cutoff (@{$job->{quantification_cutoffs}}) {
        unless ($cutoff->{labels}) {
            logdie "missing quantification cutoff labels";
        }
        unless ($cutoff->{low}) {
            logdie "missing quantification cutoff low limit";
        }
        unless ($cutoff->{high}) {
            logdie "missing quantification cutoff high limit";
        }
        unless ($cutoff->{labels} =~ m{\w+/\w+}) {
            logdie "invalid quantification cutoff labels $cutoff->{labels}\n" .
              "Labels must be of form X/Y, where X and Y are case and control labels";
        }
    }

    # Validate modifications.
    if (not defined($job->{modifications})) {
        $job->{modifications} = $Configuration::DEFAULT_MODIFICATIONS;
    }
    if (my @unknown_mods = grep !/acetyl|phospho/, @{$job->{modifications}}) {
        logdie "unknown modifications @unknown_mods given";
    }

    # Set up logging.
    if ($job->{log_file}) {
        open my $logf, ">", $job->{log_file}
          or logdie "cannot open log file $job->{log_file}: $!\n";
        $FIMM::Log::LOG = $logf;
    }
    $FIMM::Log::log_level{default} = 0 unless $job->{verbose};

    saylog "executing job file $filename";
    log_job_configuration $job;
    return $job;
}

sub log_job_configuration {
    my $job = shift;

    saylog "looking for modifications: @{$job->{modifications}}";
    saylog "looking for modified amino acids: $job->{acids}";
    while (my ($label, $cutoff) = each %{$job->{cutoffs}}) {
        saylog "using cutoff $cutoff for $label";
    }

    foreach my $cutoff (@{$job->{quantification_cutoffs}}) {
        saylog "using quantification cutoff [$cutoff->{low}, $cutoff->{high}]" .
          " for labels $cutoff->{labels}";
    }

    $job->{output_prefix} && saylog "using output prefix $job->{output_prefix}";
    if ($job->{report_prefix}) {
        saylog "using report prefix $job->{report_prefix}";
    } else {
        logdie "missing report prefix";
    }
}

1
