package PhosFox::Input;

use strict;
use warnings;

use FIMM::BioIO;
use FIMM::Log qw(debug saylog);

=head1 NAME

PhosFox::Input - process PhosFox input data

=head1 DESCRIPTION

This module contains subroutines that read in and process the given
peptide lists.  This module is part of I<PhosFox>; it is not meant for
general use.  Hence, it does not export anything.  This POD explains
how PhosFox handles the input and how the data structures are laid
out.

=head1 FUNCTIONS

=head2 open_peptide_lists $inputs, $querycols

Open peptide list files in $inputs by calling factory function
open_peptide_list() in L<FIMM::BioIO>.

=cut

sub open_peptide_lists {
    my ($inputs, $qcols) = @_;
    my @lists;

    foreach my $input (@$inputs) {
        saylog "opening peptide list file $input->{filename}";
        my ($list, $error) =
          FIMM::BioIO::open_peptide_list($input->{filename},
                                         @$qcols);
        if (not $list) {
            saylog($error, "warning");
            next;
        } else {
            saylog("peptide list file $input->{filename} recognized as " .
                   $list->type() . " list");
        }

        push(@lists, { list => $list,
                       filename => $input->{filename},
                       type => $input->{type},
                       engine => $input->{search_engine} });
    }

    return @lists;
}

=head2 process_peptides $list, $type, $engine, $cutoffs, $qcutoffs, $sequences, $acids

Process post-translationally modified peptides from PeptideList object
$list.  $type is the type of the sample from which $list was produced;
$type must be either C<case>, C<control>, or C<quantified>.

Peptides with a scores less than $cutoffs are ignored.  $cutoffs must
be a hashref whose keys are cutoff labels and values are the
corresponding cutoff values.

The function uses a FASTA hash $sequences (see read_fasta() in
L<FIMM::BioIO>) to match peptides into reference sequences.

Only modified $acids are recognized (single letters).  If there are no
modifications with accepted acids in a peptide, that peptide is
ignored.

Returns a hashref that is indexed by a UniProt accession id, and the
values are listrefs containing phosphorylated peptides detected for
that particular protein.  Peptides themselves are hashes laid out as
follows:

  $peptide = {
    id => SCALAR,               # protein identifier
    start => INTEGER,           # start locus of the peptide
    end => INTEGER,             # end locus of the peptide
    case => BOOLEAN,            # peptide identified in case sample
    control => BOOLEAN,         # peptide identified in control sample
    source => INTEGER,          # source search engine
    mods => {                   # post-translational modifications
      phospho => {
        sites => [ INTEGER... ] # phosphorylated sites
        acids => [ ACID... ]    # phosphorylated acids
      acetyl =>
        sites => [ INTEGER... ] # acetylated sites
        acids => [ ACID... ]    # acetylated acids
    }
  }

All sites are one-based and absolute with respect to the corresponding
UniProt reference sequence.

=cut

sub process_peptides {
    my ($list, $type, $engine,
        $cutoffs, $quant_cutoffs,
        $sequences, $accepted_acids) = @_;
    my %peptides;

  LINE: while (my $pep = $list->next()) {
        my $location = $list->filename() . ":" . $list->line();
        my $ids = $pep->{accessions};
        my $seq = $pep->{sequence};
        my $scores = $pep->{query_cols};
        my $mods = $pep->{mods};

        # Check cutoffs.
        while (my ($key, $value) = each %$scores) {
            if ($cutoffs->{$key} && $value < $cutoffs->{$key}) {
                debug("@$ids at $location has/have score "
                      . "$value < $cutoffs->{$key} for $key, skipping");
                next LINE;
            }
        }

        # Skip peptides with no PTMs.
        if (not $mods) {
            debug "skipping non-PTM peptide at $location";
            next;
        }

        # Determine case/control status.
        my $in_case = 0;
        my $in_control = 0;
        if ($type eq "case") { $in_case = 1; }
        if ($type eq "control") { $in_control = 1; }
        if ($type eq "quantified") {
            foreach my $qc (@$quant_cutoffs) {
                if ($scores->{$qc->{labels}}
                    && $scores->{$qc->{labels}} < $qc->{low}) {
                    $in_control = 1;
                }
                if ($scores->{$qc->{labels}}
                    && $scores->{$qc->{labels}} > $qc->{high}) {
                    $in_case = 1;
                }
                if ($scores->{$qc->{labels}}
                    && $scores->{$qc->{labels}} >= $qc->{low}
                    && $scores->{$qc->{labels}} <= $qc->{high}) {
                    $in_case = $in_control = 1;
                }
            }
        }
        unless ($in_case or $in_control) {
            saylog("$seq cannot be determined to be in either case or "
                   . "control sample at $location, skipping");
            next;
        }

        my @modlabels = keys (%$mods);
        debug("processing peptide location: $location ids: @$ids case: $in_case control: $in_control mods: @modlabels");

        # Matching against the reference sequence(s).
        foreach my $id (@$ids) {
            if (not $sequences->{$id}) {
                saylog("no sequence for $id at $location, skipping",
                       "warning");
                next;
            }

            my $peptide = { };
            $peptide->{case} = $in_case;
            $peptide->{control} = $in_control;
            $peptide->{case} || $peptide->{control}
              or die <<'EOF';
Peptide not case nor control.  This should never happen.
Please send a bug report to <petterih@iki.fi>
EOF

            my @matches = ($sequences->{$id}->{sequence} =~ /$seq/g);
            if (not @matches) {
                saylog("no match for $seq at $location, skipping",
                       "warning");
                next;
            }

            # Use one-based indexing for consistency with the rest of
            # the jungle.
            my $begin = $-[0] + 1;
            my $end = $+[0] + 1;
            if (@matches > 1) {
                saylog("multiple (" . @matches . ") matches for " .
                       "$seq at $location", "warning");
            }

            $peptide->{id} = $id;
            $peptide->{begin} = $begin;
            $peptide->{end} = $end;
            $peptide->{engine} = $engine;

            # Filter out unwanted acids for each PTM.
            my $has_accepted_ptms = 0;
            foreach my $mod (keys %$mods) {
                my ($sites, $acids) = grep_acids($sequences, $id,
                                                 $accepted_acids,
                                                 $begin, $mods->{$mod});
                if (not @$acids) {
                    debug("peptide at $location does not have any accepted " .
                          "acids for PTM $mod, skipping");
                    next;
                }

                die unless @$sites == @$acids; # sanity check
                $peptide->{mods}->{$mod}->{sites} = $sites;
                $peptide->{mods}->{$mod}->{acids} = $acids;
                $has_accepted_ptms = 1;
            }

            if ($has_accepted_ptms) {
                push @{$peptides{$id}}, $peptide;
            } else {
                debug("peptide at $location does not have any accepted " .
                      "acids in any PTM, skipping");
            }
        }
    }

    return \%peptides;
}

=head2 read_databases $filename, $mods

Read known modifications from database file $filename and adds them
into hashref $mods.  $mods is structured like this:

  $mods = {
    accession => {              # protein accession id
      phospho => [ site... ]    # known phosphorylation sites
      acetyl => [ site... ]     # known acetylation sites
    }
  }

All sites are one-based and absolute with respect to the corresponding
UniProt reference sequence.

Returns the number of modifications read.

=cut

sub read_databases {
    my ($filename, $mods) = @_;

    my $db;
    unless (open $db, "<", $filename) {
        saylog("cannot open database file $filename: $!",
               "warning");
        return;
    }

    my $n = 0;
    while (<$db>) {
        chomp;
        my ($acc, $type, $loc) = split /\t/;
        unless ($acc && $type && $loc) {
            saylog("invalid modification line $. in database file $filename",
                   "warning");
            next;
        }
        push @{$mods->{$acc}->{$type}}, $loc;
        $n++;
    }

    return $n;
}

sub grep_acids {
    my ($sequences, $id, $accepted_acids, $begin, $offsets) = @_;
    my (@sites, @acids);
    foreach (@$offsets) {
        my $acid = substr($sequences->{$id}->{sequence},
                          $begin + $_ - 2, 1);
        if ($accepted_acids =~ /$acid/) {
            push @sites, $begin + $_ - 1;
            push @acids, $acid;
        }
    }
    return \@sites, \@acids;
}

=head1 SEE ALSO

phosfox(1), L<FIMM::BioIO>

=cut

1
