package PhosFox::Output;

use strict;
use CGI qw(:standard);

use FIMM::Log qw(debug logdie);

my $outputf;
my $filehandles;

sub initialize {
    my $output_filename = shift;
    $outputf = *STDOUT;
    if ($output_filename) {
        open $outputf, ">", $output_filename
          or logdie("cannot open output file '$output_filename': $!");
    }
}

sub print_html_header {
    my ($jobname, $engines, $acids) = @_;
    my $title = $jobname || "Peptide analysis";
    $title .= " (" . localtime() . ")";
    print $outputf start_html(-encoding => "UTF-8", -title => $title), "\n";
    print $outputf CGI::start_table({ -border => 1 }), "\n";
    my $engspan = @$engines;
    my $acidspan = @$acids;
    print $outputf Tr(th({ -rowspan => 2 }, [ "Protein id", "Protein name", "Protein description",
                                              "Peptide sequence", "Modification sites" ]),
                      th({ -rowspan => 1, -colspan => $engspan }, [ "Cases", "Controls" ]),
                      th({ -rowspan => 2 }, [ "Unique case", "Unique control" ]),
                      th({ -rowspan => 1, -colspan => $acidspan }, [ "Total modifications" ])),
                        "\n";
    print $outputf Tr(th([ @$engines, @$engines, @$acids ])), "\n";
}

sub print_html_clusters {
    my ($sequences, $id, $clusters, $engines, $all_acids,
        $mod, $known_mods) = @_;

    foreach my $cluster (@$clusters) {
        my $sites = $cluster->{sites};
        my $acids = $cluster->{acids};
        my $begin = $cluster->{begin};
        my $end = $cluster->{end};
        my $unique_case = $cluster->{unique_case};
        my $unique_control = $cluster->{unique_control};

        # Calculate acid counts
        my @acid_counts;
        foreach my $acid (@$all_acids) {
            my $nacids = grep /$acid/, @$acids;
            push @acid_counts, $nacids;
        }

        my $color = "Black";
        if ($unique_case) { $color = "FireBrick"; }
        if ($unique_control) { $color = "RoyalBlue"; }

        # Render peptide string with modifications.  All indices are
        # one-based.
        my $peptide = CGI::start_tt();
        my $index = $begin;
        for my $site (@$sites) {
            $peptide .= substr($sequences->{$id}->{sequence},
                               $index - 1,
                               $site - $index);
            my $style = "text-decoration: underline";
            if ($known_mods && !contains($site, $known_mods->{$id}->{$mod})) {
                $style = "font-weight: bold; text-decoration: underline";
            }
            $peptide .= span({ -style => $style },
                             substr($sequences->{$id}->{sequence},
                                    $site - 1, 1));
            $index = $site + 1;
        }
        unless ($index > $end) {
            $peptide .= substr($sequences->{$id}->{sequence},
                               $index - 1,
                               $end - $index);
        }
        $peptide .= CGI::end_tt();

        my $modstr;
        foreach (0..$#$sites) {
            $modstr .= " " if ($_ > 0);
            if ($known_mods && !contains($$sites[$_],
                                      $known_mods->{$id}->{$mod})) {
                $modstr .= span({ -style => "font-weight: bold" },
                                "$$sites[$_]($$acids[$_])");
            } else {
                $modstr .= "$$sites[$_]($$acids[$_])";
            }
        }

        my @case_counts;
        my @control_counts;
        foreach my $engine (@$engines) {
            push @case_counts, $cluster->{cases}->{$engine} || 0;
            push @control_counts, $cluster->{controls}->{$engine} || 0;
        }

        print $outputf Tr({ -style => "color: $color" },
                          [ td([ $id,
                                 $sequences->{$id}->{name},
                                 $sequences->{$id}->{description},
                                 $peptide,
                                 $modstr,
                                 @case_counts,
                                 @control_counts,
                                 $unique_case,
                                 $unique_control,
                                 @acid_counts ])
                          ]), "\n";
    }
}

sub print_html_footer {
    print $outputf CGI::end_table(), "\n";
    print $outputf end_html, "\n";
}

sub write_peptides_and_proteins {
    my ($sequences, $id, $clusters,
        $engines, $mod, $known_mods, $prefix) = @_;

    unless ($filehandles->{$mod}) {
        foreach (@$engines) {
            open $filehandles->{$mod}->{$_}->{case_peptides}, ">", "$prefix-$_-case-peptides"
              or logdie "cannot open output file '$prefix-$_-case-peptides': $!";
            open $filehandles->{$mod}->{$_}->{case_proteins}, ">", "$prefix-$_-case-proteins"
              or logdie "cannot open output file '$prefix-$_-case-proteins': $!";
            open $filehandles->{$mod}->{$_}->{control_peptides}, ">", "$prefix-$_-control-peptides"
              or logdie "cannot open output file '$prefix-$_-control-peptides': $!";
            open $filehandles->{$mod}->{$_}->{control_proteins}, ">", "$prefix-$_-control-proteins"
              or logdie "cannot open output file '$prefix-$_-control-proteins': $!";
        }
    }

    # These hashes are used to determine whether the protein is
    # uniquely modified.  Hashes are indexed by search engines.
    my %unique_case_protein;
    my %unique_control_protein;

    foreach my $cluster (@$clusters) {
        my $sites = $cluster->{sites};
        my $acids = $cluster->{acids};
        my $begin = $cluster->{begin};
        my $end = $cluster->{end};

        my @sites_and_acids;
        foreach (0..$#$sites) {
            push @sites_and_acids, "$$sites[$_]$$acids[$_]";
        }

        foreach my $engine (@$engines) {
            if ($cluster->{cases}->{$engine} > 0) {
                print { $filehandles->{$mod}->{$engine}->{case_peptides} } "$id $begin @sites_and_acids $end\n";
            }
            if ($cluster->{controls}->{$engine} > 0) {
                print { $filehandles->{$mod}->{$engine}->{control_peptides} } "$id $begin @sites_and_acids $end\n";
            }
            if ($cluster->{cases}->{$engine} > 0 && $cluster->{controls}->{$engine} == 0) {
                $unique_case_protein{$engine} = 1;
            }
            if ($cluster->{cases}->{$engine} == 0 && $cluster->{controls}->{$engine} > 0) {
                $unique_control_protein{$engine} = 1;
            }
        }
    }

    foreach my $engine (@$engines) {
        if ($unique_case_protein{$engine}) {
            print { $filehandles->{$mod}->{$engine}->{case_proteins} } "$id\n";
        }
        if ($unique_control_protein{$engine}) {
            print { $filehandles->{$mod}->{$engine}->{control_proteins} } "$id\n";
        }
    }
}

sub contains {
    my ($item, $list) = @_;
    return 0 if not $list;
    foreach my $i (@$list) {
        return 1 if $i eq $item;
    }
    return 0;
}

1
